﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;

namespace Kooboo.Web.Mvc
{
    public interface ISelectListDataSource
    {
        IEnumerable<SelectListItem> GetSelectListItems(ViewContext viewContext);
    }
    public static class SelectListExtensions
    {
        public static IEnumerable<SelectListItem> SetActiveItem(this IEnumerable<SelectListItem> listItems, object value)
        {
            string strValue = value == null ? "" : value.ToString();
            foreach (var item in listItems)
            {
                if (string.Compare(item.Value, strValue, true) == 0)
                {
                    item.Selected = true;
                }
                yield return item;
            }

        }
    }

    public class EmptySelectListDataSource : ISelectListDataSource
    {
        #region IDropDownListDataSource Members

        public IEnumerable<SelectListItem> GetSelectListItems(ViewContext viewContext)
        {
            return new SelectListItem[] { };
        }

        #endregion
    }
    public class CultureSelectListDataSource : ISelectListDataSource
    {
        #region ISelectListDataSource Members

        public IEnumerable<SelectListItem> GetSelectListItems(ViewContext viewContext)
        {
            var cultures = System.Globalization.CultureInfo.GetCultures(System.Globalization.CultureTypes.SpecificCultures);
            foreach (var c in cultures.OrderBy(c => c.DisplayName))
            {
                yield return new SelectListItem() { Text = c.DisplayName, Value = c.Name };
            }
        }

        #endregion
    }
    public class EnumTypeSelectListDataSource : ISelectListDataSource
    {
        public EnumTypeSelectListDataSource(Type enumType)
        {
            EnumType = enumType;
        }
        public Type EnumType { get; private set; }
        #region ISelectListDataSource Members

        public IEnumerable<SelectListItem> GetSelectListItems(ViewContext viewContext)
        {
            foreach (var item in Enum.GetValues(EnumType))
            {
                yield return new SelectListItem() { Text = item.ToString(), Value = Enum.GetName(EnumType, item) };
            }
        }

        #endregion
    }
}
