﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;


namespace Kooboo.Web.Mvc.Menu
{
    public class Menu : IMenuItemContainer, ICloneable
    {
        static Menu staticMenu = null;

        public Menu()
        {
            Visible = true;
            Items = new MenuItems();
        }

        static Menu()
        {
            staticMenu = new Menu();
            Configuration.MenuSection menuSection = Configuration.MenuSection.GetSection();
            if (menuSection != null)
            {
                staticMenu.Items = CreateItems(menuSection.Items, new MenuItems());
            }
        }

        static IMenuItems CreateItems(Configuration.MenuItemElementCollection itemElementCollection, MenuItems items)
        {
            if (itemElementCollection != null)
            {
                if (!string.IsNullOrEmpty(itemElementCollection.Type))
                {
                    return (IMenuItems)Activator.CreateInstance(Type.GetType(itemElementCollection.Type));
                }
                else
                {
                    foreach (Configuration.MenuItemElement element in itemElementCollection)
                    {
                        MenuItem item = new MenuItem()
                        {
                            Text = element.Text,
                            Action = element.Action,
                            Controller = element.Controller,
                            Visible = element.Visible,
                            RouteValues = new System.Web.Routing.RouteValueDictionary(element.RouteValues.Attributes),
                            HtmlAttributes = new System.Web.Routing.RouteValueDictionary(element.HtmlAttributes.Attributes)
                        };
                        items.Add(item);
                        MenuItems subItems = new MenuItems();
                        if (element.Items != null)
                        {
                            item.Items = CreateItems(element.Items, subItems);
                        }
                    }
                }
            }
            return items;

        }

        public IMenuItemContainer Initialize(ControllerContext controllerContext)
        {
            foreach (var item in Items)
            {
                item.Initialize(controllerContext);

                if (item.IsActive)
                {
                    CurrentItem = item;
                }
            }
            return this;
        }

        public static Menu BuildMenu(ControllerContext controllerContext)
        {
            Menu menu = (Menu)staticMenu.Clone();
            menu.Initialize(controllerContext);
            return menu;
        }

        public IMenuItems Items { get;  set; }
        public MenuItem CurrentItem { get; private set; }

        #region ICloneable Members

        public object Clone()
        {
            Menu menu = new Menu();
            if (this.Items != null)
            {
                var subItems = new MenuItems();
                foreach (var item in this.Items)
                {
                    subItems.Add((MenuItem)item.Clone());
                }
                menu.Items = subItems;
            }

            return menu;
        }
        #endregion

        #region IMenuItemContainer Members

        public bool Visible
        {
            get;
            set;
        }

        #endregion
    }
}
