﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using System.Web.Routing;

namespace Kooboo.Web.Mvc.Menu
{
    public class MenuItem : IMenuItemContainer, ICloneable
    {
        public MenuItem()
        {
            Items = new MenuItems();
        }
        public virtual IMenuItemContainer Initialize(ControllerContext controllerContext)
        {
            var isActive =
               GetIsActive(controllerContext);

            foreach (var sub in this.Items)
            {
                sub.Initialize(controllerContext);
                isActive = isActive || sub.IsActive;
            }

            this.IsActive = isActive;

            if (!this.IsActive)
            {
                this.IsActive = DefaultActive(controllerContext);
            }

            return this;
        }

        protected virtual bool DefaultActive(ControllerContext controllerContext)
        {
            return string.Compare(controllerContext.RouteData.Values["controller"].ToString(), this.Controller, true) == 0;
        }

        protected virtual bool GetIsActive(ControllerContext controllerContext)
        {
            return string.Compare(controllerContext.RouteData.Values["controller"].ToString(), this.Controller, true) == 0
                           && string.Compare(controllerContext.RouteData.Values["action"].ToString(), this.Action, true) == 0;
        }

        public string Text { get; set; }
        public string Action { get; set; }
        public string Controller { get; set; }
        public bool Visible { get; set; }

        public RouteValueDictionary RouteValues { get; set; }
        public RouteValueDictionary HtmlAttributes { get; set; }

        public bool IsActive { get; set; }

        public virtual IMenuItems Items { get; set; }

        #region ICloneable Members

        public object Clone()
        {
            var item = (MenuItem)this.MemberwiseClone();
            var subItems = new MenuItems();
            if (this.Items != null)
            {
                foreach (var subItem in this.Items)
                {
                    subItems.Add((MenuItem)subItem.Clone());
                }
            }
            item.Items = subItems;
            return item;
        }

        #endregion
    }
}
